"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = require("react");

var _logger = _interopRequireDefault(require("../lib/logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var DEFAULT_BASE_URL_COOKIE_NAME = 'next-auth.base-url';
var DEFAULT_SITE = '';
var DEFAULT_BASE_PATH = '/api/auth';

var getSession = function () {
  var _ref = _asyncToGenerator(function* () {
    var {
      req
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var baseUrl = _baseUrl({
      req
    });

    var options = req ? {
      headers: {
        cookie: req.headers.cookie
      }
    } : {};
    return _fetchData("".concat(baseUrl, "/session"), options);
  });

  return function getSession() {
    return _ref.apply(this, arguments);
  };
}();

var getProviders = function () {
  var _ref2 = _asyncToGenerator(function* () {
    var {
      req
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var baseUrl = _baseUrl({
      req
    });

    var options = req ? {
      headers: {
        cookie: req.headers.cookie
      }
    } : {};
    return _fetchData("".concat(baseUrl, "/providers"), options);
  });

  return function getProviders() {
    return _ref2.apply(this, arguments);
  };
}();

var getCsrfToken = function () {
  var _ref3 = _asyncToGenerator(function* () {
    var {
      req
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var baseUrl = _baseUrl({
      req
    });

    var options = req ? {
      headers: {
        cookie: req.headers.cookie
      }
    } : {};
    var data = yield _fetchData("".concat(baseUrl, "/csrf"), options);
    return data.csrfToken;
  });

  return function getCsrfToken() {
    return _ref3.apply(this, arguments);
  };
}();

var SessionContext = (0, _react.createContext)();

var useSession = session => {
  var value = (0, _react.useContext)(SessionContext);

  if (value === undefined) {
    return useSessionData(session);
  }

  return value;
};

var useSessionData = session => {
  var [data, setData] = (0, _react.useState)(session);
  var [loading, setLoading] = (0, _react.useState)(true);

  var _getSession = function () {
    var _ref4 = _asyncToGenerator(function* () {
      try {
        setData(yield getSession());
        setLoading(false);
      } catch (error) {
        _logger.default.error('CLIENT_USE_SESSION_ERROR', error);
      }
    });

    return function _getSession() {
      return _ref4.apply(this, arguments);
    };
  }();

  (0, _react.useEffect)(() => {
    _getSession();
  }, []);
  return [data, loading];
};

var signin = function () {
  var _ref5 = _asyncToGenerator(function* (provider, args) {
    if (!provider) {
      var baseUrl = _baseUrl();

      window.location = "".concat(baseUrl, "/signin?callbackUrl=").concat(encodeURIComponent(window.location));
      return;
    }

    var providers = yield getProviders();

    if (!providers[provider]) {
      var _baseUrl2 = _baseUrl();

      window.location = "".concat(_baseUrl2, "/signin?callbackUrl=").concat(encodeURIComponent(window.location));
    } else if (providers[provider].type === 'oauth') {
      window.location = "".concat(providers[provider].signinUrl, "?callbackUrl=").concat(encodeURIComponent(window.location));
    } else {
      var options = {
        method: 'post',
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded'
        },
        body: _encodedForm(_objectSpread({
          csrfToken: yield getCsrfToken(),
          callbackUrl: window.location
        }, args))
      };
      var res = yield fetch(providers[provider].signinUrl, options);
      window.location = res.url ? res.url : window.location;
    }
  });

  return function signin(_x, _x2) {
    return _ref5.apply(this, arguments);
  };
}();

var signout = function () {
  var _ref6 = _asyncToGenerator(function* () {
    var baseUrl = _baseUrl();

    var options = {
      method: 'post',
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded'
      },
      body: _encodedForm({
        csrfToken: yield getCsrfToken(),
        callbackUrl: window.location
      })
    };
    var res = yield fetch("".concat(baseUrl, "/signout"), options);
    window.location = res.url ? res.url : window.location;
  });

  return function signout() {
    return _ref6.apply(this, arguments);
  };
}();

var Provider = (_ref7) => {
  var {
    children,
    session
  } = _ref7;
  var value = useSession(session);
  return (0, _react.createElement)(SessionContext.Provider, {
    value
  }, children);
};

var _fetchData = function () {
  var _ref8 = _asyncToGenerator(function* (url, options) {
    try {
      var res = yield fetch(url, options);
      var data = yield res.json();
      return Object.keys(data).length > 0 ? data : null;
    } catch (error) {
      _logger.default.error('CLIENT_FETCH_ERROR', url, error);

      return null;
    }
  });

  return function _fetchData(_x3, _x4) {
    return _ref8.apply(this, arguments);
  };
}();

var _baseUrl = function _baseUrl() {
  var {
    req
  } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (req) {
    var cookies = req ? _parseCookies(req.headers.cookie) : null;
    var baseUrlCookieName = process.env.NEXTAUTH_BASE_URL_COOKIE_NAME || DEFAULT_BASE_URL_COOKIE_NAME;
    var cookieValue = cookies["__Secure-".concat(baseUrlCookieName)] || cookies[baseUrlCookieName];
    var [baseUrl] = cookieValue ? cookieValue.split('|') : [null];
    return baseUrl;
  } else {
    var site = process.env.NEXTAUTH_SITE || DEFAULT_SITE;
    var basePath = process.env.NEXTAUTH_BASE_PATH || DEFAULT_BASE_PATH;
    return "".concat(site).concat(basePath);
  }
};

var _parseCookies = string => {
  if (!string) {
    return {};
  }

  try {
    var object = {};
    var a = string.split(';');

    for (var i = 0; i < a.length; i++) {
      var b = a[i].split('=');

      if (b[0].length > 1 && b[1]) {
        object[b[0].trim()] = decodeURIComponent(b[1]);
      }
    }

    return object;
  } catch (error) {
    _logger.default.error('CLIENT_COOKIE_PARSE_ERROR', error);

    return {};
  }
};

var _encodedForm = formData => {
  return Object.keys(formData).map(key => {
    return encodeURIComponent(key) + '=' + encodeURIComponent(formData[key]);
  }).join('&');
};

var _default = {
  session: getSession,
  providers: getProviders,
  csrfToken: getCsrfToken,
  getSession,
  getProviders,
  getCsrfToken,
  useSession,
  Provider,
  signin,
  signout
};
exports.default = _default;